﻿// Copyright (c) 2019 by E. Lücke

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.InteropServices;

namespace G_Api_Wrapper
{
    /// <summary>
    /// This class performs function calls to the g_api.dll file.
    /// The library implements bus independant call to Goepel hardware.
    /// </summary>
    public class GAPI
    {

        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        // common parameters
        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        /// <summary>
        /// Common return code indicating successful execution.
        /// </summary>
        public static UInt32 G_NO_ERROR = 0;


        //**********************************************************************************************************************************************************************
        // USER FUNCTIONS
        //**********************************************************************************************************************************************************************

        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        // G_GetLastErrorDescription
        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        /// <summary>
        /// G_GetLastErrorDescription
        /// Get error description of last error.
        /// </summary>
        /// <returns>string with error description</returns>
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------


        [DllImport("g_api.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr G_GetLastErrorDescription();

        public static String Call_G_GetLastErrorDescription()
        {
            IntPtr ptr = G_GetLastErrorDescription();
            return GAPI_Helpers.PtrToStringUtf8(ptr);
        }

        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        // G_GetApiVersion
        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        /// <summary>
        /// G_GetApiVersion
        /// Returns the current version of the G-API.
        /// </summary>
        /// <param name="major">returns major version</param>
        /// <param name="minor">returns minor version</param>
        /// <param name="revision">returns revision</param>
        /// <returns>G_NO_ERROR on success, G_ERROR__DLL__API__... on error</returns>
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------


        [DllImport("g_api.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 G_GetApiVersion(ref UInt32 major, ref UInt32 minor, ref UInt32 revision);

        public static UInt32 Call_G_GetApiVersion(ref UInt32 major, ref UInt32 minor, ref UInt32 revision)
        {
            return G_GetApiVersion(ref major, ref minor, ref revision);
        }


    }
}
