﻿// Copyright (c) 2019 by E. Lücke

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.InteropServices;

namespace G_Api_Wrapper
{
    /// <summary>
    /// This class performs function calls to the g_api_common.dll file.
    /// The library provides control over common properties.
    /// </summary>
    public class GAPI_Common
    {

        //**********************************************************************************************************************************************************************
        // Common types and definitions
        //**********************************************************************************************************************************************************************

        [StructLayout(LayoutKind.Sequential)]
        public struct G_Common_Signal_t {
            public Byte Size;
            public Byte ByteOrder;
            public UInt16 Offset;
        }


        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        // G_Common_OpenInterface
        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        /// <summary>
        /// G_Common_OpenInterface
        /// </summary>
        /// <description>connect interface</description>
        /// <param name="interfaceName">user device name</param>
        /// <param name="portHandle">porthandle</param>
        /// <returns>G_NO_ERROR on success, G_ERROR__DLL__COMMON__... on error</returns>
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------


        [DllImport("g_api_common.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 G_Common_OpenInterface(StringBuilder interfaceName, ref UInt32 portHandle);

        public static UInt32 Call_G_Common_OpenInterface(StringBuilder interfaceName, ref UInt32 portHandle)
        {
            return G_Common_OpenInterface(interfaceName, ref portHandle);
        }


        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        // G_Common_CloseInterface
        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        /// <summary>
        /// G_Common_CloseInterface
        /// </summary>
        /// <description>close connection to interface</description>
        /// <param name="portHandle">porthandle</param>
        /// <returns>G_NO_ERROR on success, G_ERROR__DLL__COMMON__... on error</returns>
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------


        [DllImport("g_api_common.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 G_Common_CloseInterface(UInt32 portHandle);

        public static UInt32 Call_G_Common_CloseInterface(UInt32 portHandle)
        {
            return G_Common_CloseInterface(portHandle);
        }


        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        // G_Common_InitInterface
        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        /// <summary>
        /// G_Common_InitInterface
        /// </summary>
        /// <description>initialises interface</description>
        /// <param name="portHandle">porthandle</param>
        /// <returns>G_NO_ERROR on success, G_ERROR__DLL__COMMON__... on error</returns>
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------


        [DllImport("g_api_common.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 G_Common_InitInterface(UInt32 portHandle);
       
        public static UInt32 Call_G_Common_InitInterface(UInt32 portHandle)
        {
            return G_Common_InitInterface(portHandle);
        }


        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        // G_Common_GetControllerInfo
        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        // parameters
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------

        [StructLayout(LayoutKind.Sequential)]
        public struct Interface_t
        {
            public byte InterfaceType;
            public byte InterfaceNumber;
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct G_Common_GetControllerInfo_Rsp_t
        {
            public byte ControllerFamily;
            public byte ControllerType;
            public byte reserved1;
            public byte reserved2;

            public UInt32 Flags;
            public UInt32 ResetStatusRegister;

            public UInt32 Hardwareversion;
            public UInt32 SoftwareVersion;

            public UInt32 NumberOfInterfaces;

            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 512)]
            public Interface_t[] Interfaces;

        }

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        /// <summary>
        /// G_Common_GetControllerInfo
        /// </summary>
        /// /// <description>get controller info</description>
        /// <param name="portHandle">porthandle</param>
        /// <param name="rsp">pointer to  response structure</param>
        /// <returns>G_NO_ERROR on success, G_ERROR__DLL__COMMON__... on error</returns>
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------


        [DllImport("g_api_common.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 G_Common_GetControllerInfo(UInt32 portHandle, ref G_Common_GetControllerInfo_Rsp_t rsp);

        public static UInt32 Call_G_Common_GetControllerInfo(UInt32 portHandle, ref G_Common_GetControllerInfo_Rsp_t rsp)
        {
            return G_Common_GetControllerInfo(portHandle, ref rsp);
        }

        
    }
}
