﻿// Copyright (c) 2019 by E. Lücke

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.InteropServices;


namespace G_Api_Wrapper
{
    /// <summary>
    /// This class performs function calls to the g_api_lvds.dll file.
    /// The library provides access to the LVDS hardware functionality.
    /// </summary>
    public class GAPI_LVDS
    {

        //**********************************************************************************************************************************************************************
        // FRAME GENERATOR
        //**********************************************************************************************************************************************************************

        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        // G_Lvds_FrameGenerator_GetFileInfo
        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        // parameters
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        
        public enum G_Lvds_FrameGenerator_FileType_t : Byte
        {
            UNKNOWN,
            BMP
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct G_Lvds_FrameGenerator_GetFileInfo_Rsp_t
        {
            public G_Lvds_FrameGenerator_FileType_t FileType;
            public Byte reserved;   // must be initialized with 0
            public UInt16 BitDepth; // in bit
            public UInt32 FileSize; // in byte
            public UInt32 Width;    // in px
            public UInt32 Height;   // in px
        }

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        /// <summary>
        /// G_Lvds_FrameGenerator_GetFileInfo
        /// </summary>
        /// <description>return file information</description>
        /// <param name="portHandle">portHandle</param>
        /// <param name="fileNumber">file number</param>
        /// <param name="rsp">returns file information</param>
        /// <returns>G_NO_ERROR on success, G_ERROR__DLL__LVDS__... on error</returns>
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------


        [DllImport("g_api_lvds.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 G_Lvds_FrameGenerator_GetFileInfo(UInt32 portHandle, Byte fileNumber, ref G_Lvds_FrameGenerator_GetFileInfo_Rsp_t rsp);

        public static UInt32 Call_G_Lvds_FrameGenerator_GetFileInfo(UInt32 portHandle, Byte fileNumber, ref G_Lvds_FrameGenerator_GetFileInfo_Rsp_t rsp)
        {
            return G_Lvds_FrameGenerator_GetFileInfo(portHandle, fileNumber, ref rsp);
        }


        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        // G_Lvds_FrameGenerator_GetHardwareInfo2
        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        // parameters
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------


        public enum G_Lvds_FrameGenerator_Serializer_t : Byte
        {
            UNKNOWN,
            MAX9247,
            MAX9209_MAX9213,
            DS90C241,
            INAP125T24,
            APIX_INAP375T,
            DS90UR905Q,
            DS90UB925Q,
            DS90C385A,
            MAX9275,
            MAX9273,
            MAX9271,
            DS90UB947Q,
            MAX9259,
            DS90UB913
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct  G_Lvds_FrameGenerator_GetHardwareInfo2_Rsp_t
        {
            public UInt32 HWInfoFlags;
            public UInt32 PixelClockFrequency;  // in Hz
            public UInt32 H_BlankWidth;         // in clocks
            public UInt32 H_Total;              // in clocks
            public UInt32 H_SyncWidth;          // in clocks
            public UInt32 H_FrontPorch;         // in clocks
            public UInt32 V_BlankWidth;         // in clocks
            public UInt32 V_Total;              // in clocks
            public UInt32 V_SyncWidth;          // in clocks
            public UInt32 V_FrontPorch;         // in clocks
            public UInt16 FpgaVersion;
            public G_Lvds_FrameGenerator_Serializer_t Serializer;
            public Byte reserved;               // must be initialized with 0
        }

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        /// <summary>
        /// G_Lvds_FrameGenerator_GetHardwareInfo2
        /// </summary>
        /// <description>return hardware information</description>
        /// <param name="portHandle">portHandle</param>
        /// <param name="rsp">returns hardware information</param>
        /// <returns>G_NO_ERROR on success, G_ERROR__DLL__LVDS__... on error</returns>
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------


        [DllImport("g_api_lvds.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 G_Lvds_FrameGenerator_GetHardwareInfo2(UInt32 portHandle, ref G_Lvds_FrameGenerator_GetHardwareInfo2_Rsp_t rsp);

        public static UInt32 Call_G_Lvds_FrameGenerator_GetHardwareInfo2(UInt32 portHandle, ref G_Lvds_FrameGenerator_GetHardwareInfo2_Rsp_t rsp)
        {
            return G_Lvds_FrameGenerator_GetHardwareInfo2(portHandle, ref rsp);
        }


        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        // G_Lvds_FrameGenerator_GetMemoryInfo
        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        // parameters
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
                 
        [StructLayout(LayoutKind.Sequential)]
        public struct G_Lvds_FrameGenerator_GetMemoryInfo_Rsp_t
        {
            public UInt32 Flags;
            public UInt32 MemorySize;
            public UInt32 FreeMemory;
            public UInt32 NumberOfFiles;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 256)]
            public Byte[] FileNumbers;
        }

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        /// <summary>
        /// G_Lvds_FrameGenerator_GetMemoryInfo
        /// </summary>
        /// <description>return hardware information</description>
        /// <param name="portHandle">portHandle</param>
        /// <param name="rsp">returns memory information</param>
        /// <returns>G_NO_ERROR on success, G_ERROR__DLL__LVDS__... on error</returns>
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------


        [DllImport("g_api_lvds.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 G_Lvds_FrameGenerator_GetMemoryInfo(UInt32 portHandle, ref G_Lvds_FrameGenerator_GetMemoryInfo_Rsp_t rsp);

        public static UInt32 Call_G_Lvds_FrameGenerator_GetMemoryInfo(UInt32 portHandle, ref G_Lvds_FrameGenerator_GetMemoryInfo_Rsp_t rsp)
        {
            return G_Lvds_FrameGenerator_GetMemoryInfo(portHandle, ref rsp);
        }


        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        // G_Lvds_FrameGenerator_SetDisplayProperties2
        //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        // parameters
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        
        [StructLayout(LayoutKind.Sequential)]
        public struct G_Lvds_FrameGenerator_SetDisplayProperties2_Cmd_t
        {
            public UInt32 CmdFlags;
            public UInt32 PixelClock;   // dependant on flag "USE_FREQUENCY"
            public UInt32 H_BlankWidth;
            public UInt32 H_Total;
            public UInt32 H_SyncWidth;
            public UInt32 H_FrontPorch;
            public UInt32 V_BlankWidth;
            public UInt32 V_Total;
            public UInt32 V_SyncWidth;
            public UInt32 V_FrontPorch;
        }

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        /// <summary>
        /// G_Lvds_FrameGenerator_SetDisplayProperties2
        /// </summary>
        /// <description>set display properties</description>
        /// <param name="portHandle">portHandle</param>
        /// <param name="cmd">command parameters</param>
        /// <returns>G_NO_ERROR on success, G_ERROR__DLL__LVDS__... on error</returns>
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------


        [DllImport("g_api_lvds.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 G_Lvds_FrameGenerator_SetDisplayProperties2(UInt32 portHandle, ref G_Lvds_FrameGenerator_SetDisplayProperties2_Cmd_t cmd);

        public static UInt32 Call_G_Lvds_FrameGenerator_SetDisplayProperties2(UInt32 portHandle, ref G_Lvds_FrameGenerator_SetDisplayProperties2_Cmd_t cmd)
        {
            return G_Lvds_FrameGenerator_SetDisplayProperties2(portHandle, ref cmd);
        }
        
        
    }
}
